<?php
/**
 * @brief		QR Code share link
 * @author		<a href='https://invisionbyte.net'>Terabyte - Invision Byte</a>
 * @copyright	(c) Invision Byte
 * @package		(TB) QR Code Share Service
 * @since		1.0.0
 */


namespace IPS\tbqcss\ShareServices;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * QR Code share link
 */
class _Qrcode extends \IPS\Content\ShareServices
{
	/**
	 * Add any additional form elements to the configuration form. These must be setting keys that the service configuration form can save as a setting.
	 *
	 * @param	\IPS\Helpers\Form				$form		Configuration form for this service
	 * @param	\IPS\core\ShareLinks\Service	$service	The service
	 * @return	void
	 */
	public static function modifyForm( \IPS\Helpers\Form &$form, $service )
	{
		# Additional settings
		$form->addHeader('tb_qcss_api_header');
		
		$form->add( new \IPS\Helpers\Form\Radio( 'tb_qcss_api', \IPS\Settings::i()->tb_qcss_api, FALSE, array( 'options' => array( 'google' => 'tb_qcss_api_google', 'quickchart' => 'tb_qcss_api_quickchart' ), 'toggles' => array( 'quickchart' => array( 'tb_qcss_qc_foreground', 'tb_qcss_qc_background', 'tb_qcss_qc_format' ) ) ), NULL, NULL, NULL, 'tb_qcss_api' ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'tb_qcss_size', \IPS\Settings::i()->tb_qcss_size, FALSE, array( 'step' => 1 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('px'), 'tb_qcss_size' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tb_qcss_margin', \IPS\Settings::i()->tb_qcss_margin, FALSE, array( 'step' => 1 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('tb_qcss_margin_suffix'), 'tb_qcss_margin' ) );
		$form->add( new \IPS\Helpers\Form\Color( 'tb_qcss_qc_foreground', \IPS\Settings::i()->tb_qcss_qc_foreground, FALSE, array(), NULL, NULL, NULL, 'tb_qcss_qc_foreground' ) );
		$form->add( new \IPS\Helpers\Form\Color( 'tb_qcss_qc_background', \IPS\Settings::i()->tb_qcss_qc_background, FALSE, array(), NULL, NULL, NULL, 'tb_qcss_qc_background' ) );
		$form->add( new \IPS\Helpers\Form\Select( 'tb_qcss_qc_format', \IPS\Settings::i()->tb_qcss_qc_format, FALSE, array( 'options' => array( 'png' => 'PNG', 'svg' => 'SVG' ), 'parse' => 'raw' ), NULL, NULL, NULL, 'tb_qcss_qc_format' ) );
		$form->add( new \IPS\Helpers\Form\Select( 'tb_qcss_ecl', \IPS\Settings::i()->tb_qcss_ecl, FALSE, array( 'options' => array( 'L' => 'tb_qcss_ecl_low', 'M' => 'tb_qcss_ecl_medium', 'Q' => 'tb_qcss_ecl_quartile', 'H' => 'tb_qcss_ecl_high' ) ), NULL, NULL, NULL, 'tb_qcss_ecl' ) );
		
		# Preview field
		$form->addDummy( 'tb_qcss_qrcode_preview', \IPS\Theme::i()->getTemplate( 'sharelink', 'tbqcss', 'admin' )->formApiPreview(), '', '', 'tb_qcss_qrcode_preview' );
		
		# Add our JS controller/file for the preview
		$form->attributes[' data-controller'] = 'tbqcss.admin.form.preview';
		\IPS\Output::i()->jsFiles  = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'admin_preview.js', 'tbqcss', 'admin' ) );
		
		# @TODO: Remove "Image format" option so we always use PNG for easier hangling of webshare data?
	}
	
	/**
	 * Build the QR Code image url
	 *
	 * @return	\IPS\Http\Url
	 */
	public function getQrCodeImageUrl()
	{
		# Init vars
		$qrCodeUrl = '';
		$addParams = [];
		
		/**
		 * Can't use \IPS\Http\Url::external() here because it encodes twice the data which is NOT what we want...
		 */
		if ( \IPS\Settings::i()->tb_qcss_api == 'google' )
		{
			$qrCodeUrl = 'https://chart.googleapis.com/chart';
			
			$addParams = [
				'cht'	=> 'qr',
				'chs'	=> \IPS\Settings::i()->tb_qcss_size,
				'chl'	=> urlencode( $this->url ),
				'chld'	=> \IPS\Settings::i()->tb_qcss_ecl . '|' . \IPS\Settings::i()->tb_qcss_margin
			];
		}
		elseif ( \IPS\Settings::i()->tb_qcss_api == 'quickchart' )
		{
			$qrCodeUrl = 'https://quickchart.io/qr';
			
			$addParams = [
				'size'		=> \IPS\Settings::i()->tb_qcss_size,
				'text'		=> urlencode( $this->url ),
				'margin'	=> \IPS\Settings::i()->tb_qcss_margin,
				'dark'		=> \IPS\Settings::i()->tb_qcss_qc_foreground,
				'light'		=> \IPS\Settings::i()->tb_qcss_qc_background,
				'ecLevel'	=> \IPS\Settings::i()->tb_qcss_ecl,
				'format'	=> \IPS\Settings::i()->tb_qcss_qc_format
			];
		}
		
		# Got API url?
		if ( $qrCodeUrl AND $addParams )
		{
			# Add ?
			$qrCodeUrl .= '?';
			$qrParams   = [];
			
			foreach( $addParams as $k => $v )
			{
				# Remove # from color settings
				$v = str_replace( '#', '', $v );
				
				# Add parsed parameter to array
				$qrParams[] = "{$k}={$v}";
			}
			
			# Get final API url
			$qrCodeUrl .= implode( '&', $qrParams );
		}
		
		# Return url
		return $qrCodeUrl;
	}

	/**
	 * Return the HTML code to show the share link
	 *
	 * @return	string
	 */
	public function __toString()
	{
		return \IPS\Theme::i()->getTemplate( 'sharelink', 'tbqcss' )->qrcodeIcon( $this->getQrCodeImageUrl() );
	}
}